import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)

import pandas as pd
import mns_common.utils.date_handle_util as date_handle_util
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.api.em.east_money_stock_api as east_money_stock_api
from loguru import logger
import threading
import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api
import mns_common.component.common_service_fun_api as common_service_fun_api
import mns_scheduler.k_line.sync.daily_week_month_line_sync as daily_week_month_line_sync_api
import mns_scheduler.k_line.clean.k_line_info_clean_impl as k_line_info_clean_impl
import mns_common.utils.data_frame_util as data_frame_util
import mns_common.component.company.company_common_service_api as company_common_service_api

K_LINE_CLEAN_DB_NAME = 'k_line_clean_fail_name'

# 定义一个全局锁，用于保护 result 变量的访问
result_lock = threading.Lock()
# 初始化 result 变量为一个空的 Pandas DataFrame
result = pd.DataFrame()
# 分页大小
MAX_PAGE_NUMBER = 1000
mongodb_util = MongodbUtil('27017')


def sync_k_line_info_task(str_day):
    # 创建索引
    create_k_line_index()

    last_trade_day = trade_date_common_service_api.get_last_trade_day(str_day)
    query = {'date': date_handle_util.no_slash_date(last_trade_day)}
    count = mongodb_util.count(query, 'stock_qfq_daily')
    # 当天没有k线数据时 进行同步
    if count == 0:
        daily_week_month_line_sync_api.sync_all_daily_data('daily', 'qfq', 'stock_qfq_daily', str_day,
                                                           None)
    sync_k_line_info(str_day, None)


def sync_k_line_info(str_day, symbol_list):
    result_k_line_list_df = None
    if symbol_list is not None:
        for symbol in symbol_list:
            try:
                company_df = company_common_service_api.get_company_info_industry_list_date()
                company_info_df = company_df.loc[company_df['_id'] == symbol]
                if data_frame_util.is_not_empty(company_info_df):

                    # 将日期数值转换为日期时间格式
                    company_info_df['list_date_01'] = pd.to_datetime(
                        company_info_df['list_date'],
                        format='%Y%m%d')
                    # 将日期字符串转换为日期时间格式
                    str_day_date = date_handle_util.str_to_date(str_day, '%Y-%m-%d')

                    # 计算日期差值 距离现在上市时间
                    company_info_df[
                        'diff_days'] = company_info_df.apply(
                        lambda row: (str_day_date - row['list_date_01']).days, axis=1)

                    diff_days = list(company_info_df['diff_days'])[0]

                    k_line_result = k_line_info_clean_impl.calculate_k_line_info(str_day, symbol, diff_days)
                    save_k_line_data(symbol, str_day, k_line_result)
                    if result_k_line_list_df is None:
                        result_k_line_list_df = k_line_result
                    else:
                        result_k_line_list_df = pd.concat([result_k_line_list_df, k_line_result])

            except BaseException as e:
                logger.error("k线同步错误:{},{},{}", str_day, symbol, e)
    else:
        result_k_line_list_df = multi_threaded_k_line_sync(str_day)
    logger.info("计算k线数据任务完成:{}", str_day)
    return result_k_line_list_df


def handle_fail_data(str_day, real_time_quotes_now):
    query = {'str_day': str_day}
    k_line_fail_df = mongodb_util.find_query_data(K_LINE_CLEAN_DB_NAME, query)
    if data_frame_util.is_not_empty(k_line_fail_df):
        fail_data_df = real_time_quotes_now.loc[real_time_quotes_now['symbol'].isin(k_line_fail_df['symbol'])]
        single_threaded_sync_task(fail_data_df, str_day, 88)


# 多线程同步任务
def multi_threaded_k_line_sync(str_day):
    # 退市代码
    de_list_company_symbols = company_common_service_api.get_de_list_company()
    real_time_quotes_now = east_money_stock_api.get_real_time_quotes_all_stocks()
    real_time_quotes_now = real_time_quotes_now.loc[~(real_time_quotes_now['symbol'].isin(de_list_company_symbols))]

    # 将list_date列中的所有NaN值设置为99990909
    real_time_quotes_now['list_date'].fillna(20990909.0, inplace=True)

    # 将日期数值转换为日期时间格式
    real_time_quotes_now['list_date_01'] = pd.to_datetime(real_time_quotes_now['list_date'], format='%Y%m%d')

    now_date = date_handle_util.str_to_date(str_day, '%Y-%m-%d')

    # 计算日期差值 距离现在上市时间
    real_time_quotes_now['diff_days'] = real_time_quotes_now.apply(
        lambda row: (now_date - row['list_date_01']).days, axis=1)

    #  exclude b symbol
    real_time_quotes_now = common_service_fun_api.exclude_b_symbol(real_time_quotes_now.copy())
    # exclude amount==0 symbol
    real_time_quotes_now = common_service_fun_api.exclude_amount_zero_stock(real_time_quotes_now)
    total_count = real_time_quotes_now.shape[0]
    global result
    result = pd.DataFrame()  # 重新初始化 result 变量
    threads = []
    page_number = round(total_count / MAX_PAGE_NUMBER, 0) + 1
    page_number = int(page_number)
    # 创建多个线程来获取数据
    for page in range(page_number):  # 0到page_number页
        logger.info("启动第{}个线程", page + 1)

        end_count = (page + 1) * MAX_PAGE_NUMBER
        begin_count = page * MAX_PAGE_NUMBER
        page_df = real_time_quotes_now.loc[begin_count:end_count]

        thread = threading.Thread(target=single_threaded_sync_task, args=(page_df, str_day, page))
        threads.append(thread)
        thread.start()

    # 等待所有线程完成
    for thread in threads:
        thread.join()
    # 处理失败数据
    handle_fail_data(str_day, real_time_quotes_now)
    # 返回获取的接口数据
    return result


# 单线程同步任务
def single_threaded_sync_task(page_df, str_day, page):
    global result
    for stock_one in page_df.itertuples():
        try:
            k_line_df = k_line_info_clean_impl.calculate_k_line_info(str_day, stock_one.symbol, stock_one.diff_days)
            save_k_line_data(stock_one.symbol, str_day, k_line_df)
            if k_line_df is None:
                result = k_line_df
            else:
                result = pd.concat([k_line_df, result])

        except BaseException as e:
            fail_symbol = {
                '_id': str_day + "_" + stock_one.symbol,
                'symbol': stock_one.symbol,
                'str_day': str_day}
            fail_symbol_df = pd.DataFrame(fail_symbol, index=[1])
            logger.error("k线数据清异常:{},{}", stock_one.symbol, e)
            mongodb_util.save_mongo(fail_symbol_df, K_LINE_CLEAN_DB_NAME)
    logger.info("k线数据清洗到:{}页", page + 1)
    return result


def save_k_line_data(symbol, str_day, k_line_info):
    k_line_info['_id'] = symbol + '_' + str_day
    mongodb_util.save_mongo_no_catch_exception(k_line_info, 'k_line_info')


# 创建索引
def create_k_line_index():
    mongodb_util.create_index('k_line_info', [("symbol", 1)])
    mongodb_util.create_index('k_line_info', [("str_day", 1)])
    mongodb_util.create_index('k_line_info', [("str_day", 1), ("symbol", 1)])





import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)
import mns_common.api.em.east_money_stock_hk_api as east_money_stock_hk_api
import akshare as ak
import mns_common.constant.db_name_constant as db_name_constant
from mns_common.db.MongodbUtil import MongodbUtil
from functools import lru_cache
import mns_common.api.hk.ths_hk_company_info_api as ths_hk_company_info_api
import pandas as pd

mongodb_util = MongodbUtil('27017')
from loguru import logger


# 获取陆股通的列表
@lru_cache(maxsize=None)
def get_hk_ggt_component():
    stock_hk_ggt_components_em_df = ak.stock_hk_ggt_components_em()
    stock_hk_ggt_components_em_df = stock_hk_ggt_components_em_df.rename(columns={
        "序号": "index",
        "代码": "symbol",
        "名称": "name"
    })
    return stock_hk_ggt_components_em_df


# 获取em cookie
@lru_cache(maxsize=None)
def get_em_cookie():
    query = {"type": "em_cookie"}
    stock_account_info = mongodb_util.find_query_data(db_name_constant.STOCK_ACCOUNT_INFO, query)
    cookie = list(stock_account_info['cookie'])[0]
    return cookie


@lru_cache(maxsize=None)
def get_ths_cookie():
    query = {"type": "ths_cookie"}
    stock_account_info = mongodb_util.find_query_data(db_name_constant.STOCK_ACCOUNT_INFO, query)
    cookie = list(stock_account_info['cookie'])[0]
    return cookie


# https://quote.eastmoney.com/center/gridlist.html#hk_stocks
def sync_hk_company_info():
    cookie = get_em_cookie()
    hk_real_time_df = east_money_stock_hk_api.hk_real_time_quotes(cookie)

    hk_real_time_df = hk_real_time_df[[
        "symbol",
        "name",
        "chg",
        "total_mv",
        "flow_mv",
        "list_date",
        "industry",
        "amount",
        "now_price"
    ]]
    # 排除基金
    hk_real_time_df = hk_real_time_df.loc[hk_real_time_df['total_mv'] != '-']

    stock_hk_ggt_components_em_df = get_hk_ggt_component()
    stock_hk_ggt_components_symbol_list = list(stock_hk_ggt_components_em_df['symbol'])
    hk_real_time_df['hk_ggt'] = False
    hk_real_time_df.loc[hk_real_time_df['symbol'].isin(stock_hk_ggt_components_symbol_list), 'hk_ggt'] = True
    hk_real_time_df.loc[hk_real_time_df['industry'] == '-', 'industry'] = '其他'

    hk_real_time_df['_id'] = hk_real_time_df['symbol']

    hk_real_time_df.fillna(0, inplace=True)
    mongodb_util.remove_all_data(db_name_constant.COMPANY_INFO_HK)
    hk_real_time_df = hk_real_time_df.sort_values(by=['hk_ggt'], ascending=False)
    for stock_one in hk_real_time_df.itertuples():
        try:
            symbol = stock_one.symbol
            ths_cookie = get_ths_cookie()
            company_hk_df = ths_hk_company_info_api.get_hk_company_info(symbol, ths_cookie)

            company_hk_df = company_hk_df.rename(columns={
                "industry": "industry_detail",
                "list_date": "list_date_str",
            })

            hk_real_time_one_df = hk_real_time_df.loc[hk_real_time_df['symbol'] == symbol]

            company_hk_df = company_hk_df.set_index(['symbol'], drop=True)
            hk_real_time_one_df = hk_real_time_one_df.set_index(['symbol'], drop=False)
            company_hk_df = pd.merge(company_hk_df, hk_real_time_one_df, how='outer',
                                     left_index=True, right_index=True)

            mongodb_util.save_mongo(company_hk_df, db_name_constant.COMPANY_INFO_HK)
        except BaseException as e:
            logger.error("同步港股公司信息异常:{},{}", symbol, e)


if __name__ == '__main__':
    sync_hk_company_info()



import sys
import os

import pandas as pd
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.component.company.company_common_service_api as company_common_service_api

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)
mongodb_util = MongodbUtil('27017')
import mns_common.api.msg.push_msg_api as push_msg_api
import mns_scheduler.company_info.base.sync_company_base_info_api as company_info_sync_api
import mns_scheduler.company_info.clean.company_info_clean_api as company_info_clean_api
import mns_scheduler.concept.ths.detaill.ths_concept_detail_api as ths_concept_detail_api
import mns_common.component.concept.ths_concept_common_service_api as ths_concept_common_service_api
import mns_common.component.company.company_common_service_new_api as company_common_service_new_api
import mns_common.utils.data_frame_util as data_frame_util
import mns_common.constant.db_name_constant as db_name_constant
import mns_scheduler.concept.clean.ths_concept_clean_api as ths_concept_clean_api
import mns_common.constant.redis_msg_constant as redis_msg_constant
import mns_common.component.redis_msg.redis_msg_publish_service as redis_msg_publish_service
from loguru import logger


# 推送消息
def push_msg_to_we_chat_and_redis(concept_code, concept_name, url):
    msg = "概念代码:" + str(concept_code) + "," + "概念名称:" + concept_name + "," + "url:   " + url
    title = "新增同花顺概念:" + str(concept_code) + "-" + concept_name
    # 推送到微信
    push_msg_api.push_msg_to_wechat(title, msg)
    # 项目之前推送消息
    redis_msg_publish_service.send_redis_msg(redis_msg_constant.THS_CONCEPT_MSG_TOPIC,
                                             redis_msg_constant.THS_NEW_CONCEPT_ADD_MSG)

    # 更新ths概念信息
    ths_concept_clean_api.update_ths_concept_info()


# 保存新概念信息到数据库
def save_ths_concept_list(concept_code, concept_name, str_day, str_now_time):
    url = 'http://q.10jqka.com.cn/thshy/detail/code/' + str(concept_code)
    ths_concept_list = pd.DataFrame([
        [concept_code, concept_code, concept_name, str_day, url, str_now_time, True, True],
    ], columns=['_id', 'symbol', 'name', 'str_day', 'url', 'str_now_time', 'success', 'valid'])
    mongodb_util.save_mongo(ths_concept_list, 'ths_concept_list')


# 保存新概念详细信息到数据库
def save_ths_concept_detail(new_concept_symbol_df,
                            concept_name, str_day,
                            str_now_time, concept_code):
    concept_code = int(concept_code)
    new_concept_symbol_df['symbol'] = new_concept_symbol_df['symbol'].astype(str)
    new_concept_symbol_df['_id'] = str(concept_code) + '_' + new_concept_symbol_df['symbol']
    new_concept_symbol_df['concept_code'] = concept_code
    new_concept_symbol_df['concept_name'] = concept_name

    new_concept_symbol_df['concept_name'] = new_concept_symbol_df['concept_name'].replace(" ", "")

    all_ths_concept_df = ths_concept_common_service_api.get_all_ths_concept()
    ths_concept_one_df = all_ths_concept_df.loc[all_ths_concept_df['symbol'] == int(concept_code)]

    if data_frame_util.is_empty(ths_concept_one_df):
        concept_create_day = str_day
    else:
        concept_create_day = list(ths_concept_one_df['str_day'])[0]

    new_concept_symbol_df['str_day'] = str_day
    new_concept_symbol_df['str_now_time'] = str_now_time
    new_concept_symbol_df['concept_create_day'] = concept_create_day

    new_concept_symbol_list = list(new_concept_symbol_df['symbol'])

    query_company_info = {'symbol': {'$in': new_concept_symbol_list}}
    query_company_info_key = str(query_company_info)
    query_field = {"first_industry": 1, "first_industry": 1, "industry": 1,
                   "company_type": 1, "flow_mv_sp": 1,
                   "total_mv_sp": 1}
    query_field_key = str(query_field)
    company_info = company_common_service_new_api.get_company_info_by_field(query_company_info_key, query_field_key)

    if 'industry' in new_concept_symbol_df.columns:
        del new_concept_symbol_df['industry']
    if 'company_type' in new_concept_symbol_df.columns:
        del new_concept_symbol_df['company_type']
    if 'flow_mv_sp' in new_concept_symbol_df.columns:
        del new_concept_symbol_df['flow_mv_sp']
    if 'total_mv_sp' in new_concept_symbol_df.columns:
        del new_concept_symbol_df['total_mv_sp']

    company_info = company_info.set_index(['_id'], drop=True)
    new_concept_symbol_df = new_concept_symbol_df.set_index(['symbol'], drop=False)

    new_concept_symbol_df = pd.merge(new_concept_symbol_df, company_info, how='outer',
                                     left_index=True, right_index=True)

    if 'index' not in company_info.columns:
        new_concept_symbol_df['index'] = 0

    if 'change' not in company_info.columns:
        new_concept_symbol_df['change'] = 0

    new_concept_symbol_df['concept_name'] = new_concept_symbol_df['concept_name'].replace(" ", "")

    if bool(1 - ('way' in new_concept_symbol_df.columns)):
        new_concept_symbol_df['way'] = 'symbol_sync'
    if "long" not in new_concept_symbol_df.columns:
        new_concept_symbol_df['long'] = ''
    if "short" not in new_concept_symbol_df.columns:
        new_concept_symbol_df['short'] = new_concept_symbol_df['long']
    new_concept_symbol_df = new_concept_symbol_df[[
        "_id",
        "index",
        "symbol",
        "name",
        "now_price",
        "chg",
        "change",
        "exchange",
        "amount",
        "concept_code",
        "concept_name",
        "str_day",
        "str_now_time",
        "industry",
        "flow_mv_sp",
        "total_mv_sp",
        "company_type",
        "concept_create_day",
        "way",
        "long",
        'short'
    ]]
    query_detail = {"concept_code": int(concept_code)}
    exist_concept_detail = mongodb_util.find_query_data(db_name_constant.THS_STOCK_CONCEPT_DETAIL, query_detail)
    # 更新入选理由
    update_long_short(new_concept_symbol_df)

    if exist_concept_detail is None or exist_concept_detail.shape[0] == 0:
        new_concept_symbol_df['grade'] = 1
        # 详细标识
        new_concept_symbol_df['remark'] = ''
        # 简单标识
        new_concept_symbol_df['remark_flag'] = ''
        mongodb_util.save_mongo(new_concept_symbol_df, db_name_constant.THS_STOCK_CONCEPT_DETAIL)
        # 保存到当日新增概念列表
        new_concept_symbol_df['concept_type'] = 'ths'
        mongodb_util.save_mongo(new_concept_symbol_df, db_name_constant.TODAY_NEW_CONCEPT_LIST)
    else:
        exist_concept_detail_symbol_list = list(exist_concept_detail['symbol'])
        new_concept_symbol_df = new_concept_symbol_df.loc[~(
            new_concept_symbol_df['symbol'].isin(exist_concept_detail_symbol_list))]
        if new_concept_symbol_df.shape[0] > 0:
            new_concept_symbol_df['grade'] = 1
            # 详细标识
            new_concept_symbol_df['remark'] = ''
            # 简单标识
            new_concept_symbol_df['remark_flag'] = ''
            mongodb_util.save_mongo(new_concept_symbol_df, db_name_constant.THS_STOCK_CONCEPT_DETAIL)
            # 保存到当日新增概念列表
            new_concept_symbol_df['concept_type'] = 'ths'
            mongodb_util.save_mongo(new_concept_symbol_df, db_name_constant.TODAY_NEW_CONCEPT_LIST)

    update_company_info(new_concept_symbol_df)
    # 公司缓存信息清除
    company_common_service_api.company_info_industry_cache_clear()


# 更新入选理由
def update_long_short(new_concept_symbol_df):
    if data_frame_util.is_empty(new_concept_symbol_df):
        return None
    for new_concept_one in new_concept_symbol_df.itertuples():
        try:
            query = {'symbol': new_concept_one.symbol, 'concept_code': new_concept_one.concept_code}
            new_values = {"$set":{'long': new_concept_one.long, "short": new_concept_one.short}}
            mongodb_util.update_many(query, new_values, db_name_constant.THS_STOCK_CONCEPT_DETAIL)
            mongodb_util.update_many(query, new_values, db_name_constant.THS_STOCK_CONCEPT_DETAIL_APP)
        except BaseException as e:
            logger.error("更新入选理由异常:{}", e)

        # 更新公司表信息 todo 清空cache 公司表中  common_service_fun_api.py  get_company_info_industry


def update_company_info(new_concept_symbol_df):
    if new_concept_symbol_df.shape[0] > 0:
        symbol_list = list(new_concept_symbol_df['symbol'])
        company_info_sync_api.sync_company_base_info(symbol_list)
        company_info_clean_api.fix_company_industry(None)
        # 公司缓存信息清除
        company_common_service_api.company_info_industry_cache_clear()


# 获取最大概念代码
def get_max_concept_code():
    query = {"symbol": {'$ne': 'null'}, "success": True}
    ths_concept_max = mongodb_util.descend_query(query, 'ths_concept_list', 'symbol', 1)
    if ths_concept_max.shape[0] == 0:
        concept_code = 885284
    else:
        concept_code = list(ths_concept_max['symbol'])[0]

    return concept_code


def get_concept_detail_info_web(concept_code):
    new_concept_symbol_list = ths_concept_detail_api.get_ths_concept_detail(concept_code, None)
    if new_concept_symbol_list is None or new_concept_symbol_list.shape[0] == 0:
        return None
    new_concept_symbol_list['_id'] = str(concept_code) + '-' + new_concept_symbol_list['symbol']
    return new_concept_symbol_list



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 17
project_path = file_path[0:end]
sys.path.append(project_path)
import pandas as pd
import mns_common.api.akshare.stock_zt_pool_api as stock_zt_pool_api
import mns_common.utils.date_handle_util as date_handle_util
import mns_common.component.concept.ths_concept_common_service_api as ths_concept_common_service_api
from mns_common.db.MongodbUtil import MongodbUtil
import mns_common.component.company.company_common_service_api as company_common_service_api
from loguru import logger
import mns_common.utils.data_frame_util as data_frame_util
import mns_common.component.common_service_fun_api as common_service_fun_api
import mns_common.component.trade_date.trade_date_common_service_api as trade_date_common_service_api
import mns_common.api.ths.zt.ths_stock_zt_pool_v2_api as ths_stock_zt_pool_v2_api
import mns_common.component.zt.zt_common_service_api as zt_common_service_api
import mns_common.api.em.east_money_stock_api as east_money_stock_api
from datetime import datetime

'''
东方财富涨停池
'''

mongodb_util = MongodbUtil('27017')

ZT_FIELD = ['_id', 'symbol', 'name', 'now_price', 'chg', 'first_closure_time',
            'last_closure_time', 'connected_boards_numbers',
            'zt_reason', 'closure_funds',
            # 'closure_funds_per_amount', 'closure_funds_per_flow_mv',
            'frying_plates_numbers',
            # 'statistics_detail', 'zt_type', 'market_code',
            'statistics',
            # 'zt_flag',
            'industry', 'first_sw_industry',
            'second_sw_industry',
            'third_sw_industry', 'ths_concept_name',
            'ths_concept_code', 'ths_concept_sync_day', 'em_industry',
            'mv_circulation_ratio', 'ths_concept_list_info', 'kpl_plate_name',
            'kpl_plate_list_info', 'company_type', 'diff_days', 'symbol', 'amount',
            'list_date',
            'exchange', 'flow_mv', 'total_mv',
            'classification', 'flow_mv_sp', 'total_mv_sp', 'flow_mv_level',
            'amount_level', 'new_stock', 'list_date_01', 'index', 'str_day']


def save_zt_info(str_day):
    if bool(1 - trade_date_common_service_api.is_trade_day(str_day)):
        return None

    stock_em_zt_pool_df_data = stock_zt_pool_api.stock_em_zt_pool_df(
        date_handle_util.no_slash_date(str_day))

    # fix 涨停池没有的股票
    stock_em_zt_pool_df_data = sync_miss_zt_data(stock_em_zt_pool_df_data, str_day)

    stock_em_zt_pool_df_data = common_service_fun_api.total_mv_classification(stock_em_zt_pool_df_data)

    stock_em_zt_pool_df_data = common_service_fun_api.classify_symbol(stock_em_zt_pool_df_data)

    stock_em_zt_pool_df_data = common_service_fun_api.symbol_amount_simple(stock_em_zt_pool_df_data)

    stock_em_zt_pool_df_data = company_common_service_api.amendment_industry(stock_em_zt_pool_df_data)

    # 上个交易交易日涨停股票
    last_trade_day_zt_df = zt_common_service_api.get_last_trade_day_zt(str_day)

    try:
        # 同花顺问财涨停池
        ths_zt_pool_df_data = ths_stock_zt_pool_v2_api.get_ths_stock_zt_reason_with_cache(str_day)

        # del stock_em_zt_pool_df_data['ths_concept_name']
        # del stock_em_zt_pool_df_data['ths_concept_code']

        for stock_one in stock_em_zt_pool_df_data.itertuples():
            try:
                stock_em_zt_pool_df_data = ths_concept_common_service_api.set_last_ths_concept(stock_one.symbol,
                                                                                               stock_em_zt_pool_df_data,
                                                                                               str_day)
                # 设置连板数目
                stock_em_zt_pool_df_data = set_connected_boards_numbers(stock_em_zt_pool_df_data,
                                                                        stock_one.symbol, last_trade_day_zt_df)

                ths_zt_pool_one_df = ths_zt_pool_df_data.loc[ths_zt_pool_df_data['symbol'] == stock_one.symbol]
                if data_frame_util.is_empty(ths_zt_pool_one_df):
                    stock_em_zt_pool_df_data['zt_reason'] = '0'
                    continue
                stock_em_zt_pool_df_data.loc[stock_em_zt_pool_df_data['symbol'] == stock_one.symbol, 'zt_reason'] = \
                    list(ths_zt_pool_one_df['zt_reason'])[0]
            except BaseException as e:
                stock_em_zt_pool_df_data['zt_reason'] = '0'
                logger.error("出现异常:{}", e)
    except BaseException as e:
        stock_em_zt_pool_df_data['zt_reason'] = '0'
        logger.error("出现异常:{}", e)

    stock_em_zt_pool_df_data['list_date'] = stock_em_zt_pool_df_data['list_date'].apply(
        lambda x: pd.to_numeric(x, errors="coerce"))

    stock_em_zt_pool_df_data['new_stock'] = False
    # 将日期数值转换为日期时间格式
    stock_em_zt_pool_df_data['list_date_01'] = pd.to_datetime(stock_em_zt_pool_df_data['list_date'], format='%Y%m%d')
    str_day_date = date_handle_util.str_to_date(str_day, '%Y-%m-%d')
    # 计算日期差值 距离现在上市时间
    stock_em_zt_pool_df_data['diff_days'] = stock_em_zt_pool_df_data.apply(
        lambda row: (str_day_date - row['list_date_01']).days, axis=1)
    # 上市时间小于100天为新股
    stock_em_zt_pool_df_data.loc[
        stock_em_zt_pool_df_data["diff_days"] < 100, ['new_stock']] \
        = True
    stock_em_zt_pool_df_data = stock_em_zt_pool_df_data.dropna(subset=['diff_days'], axis=0, inplace=False)

    # 按照"time"列进行排序，同时将值为0的数据排到最末尾
    stock_em_zt_pool_df_data = stock_em_zt_pool_df_data.sort_values(by=['first_closure_time'])

    # 重置索引，并将排序结果保存到新的"index"列中

    stock_em_zt_pool_df_data['str_day'] = str_day
    stock_em_zt_pool_df_data['_id'] = stock_em_zt_pool_df_data['symbol'] + "_" + str_day

    stock_em_zt_pool_df_data = stock_em_zt_pool_df_data[ZT_FIELD]
    exist_zt_pool_today = mongodb_util.find_query_data('stock_zt_pool', {'str_day': str_day})
    if data_frame_util.is_empty(exist_zt_pool_today):
        mongodb_util.save_mongo(stock_em_zt_pool_df_data, 'stock_zt_pool')
    else:
        exist_zt_pool_today_not_nan = exist_zt_pool_today.loc[exist_zt_pool_today['zt_reason'] != '0']

        stock_em_zt_pool_df_data = stock_em_zt_pool_df_data.loc[:, ~stock_em_zt_pool_df_data.columns.duplicated()]
        stock_em_zt_pool_df_data_new = stock_em_zt_pool_df_data.loc[~(
            stock_em_zt_pool_df_data['symbol'].isin(exist_zt_pool_today_not_nan['symbol']))]
        if data_frame_util.is_not_empty(stock_em_zt_pool_df_data_new):
            mongodb_util.save_mongo(stock_em_zt_pool_df_data_new, 'stock_zt_pool')
    return stock_em_zt_pool_df_data


# 设置连板数目
def set_connected_boards_numbers(stock_em_zt_pool_df_data, symbol, last_trade_day_zt_df):
    connected_boards_df = last_trade_day_zt_df.loc[
        last_trade_day_zt_df['symbol'].isin(stock_em_zt_pool_df_data['symbol'])]

    connected_boards_df['connected_boards_numbers'] = connected_boards_df['connected_boards_numbers'] + 1

    connected_boards_df_one = connected_boards_df.loc[connected_boards_df['symbol'] == symbol]
    if data_frame_util.is_not_empty(connected_boards_df_one):
        connected_boards_df_one.loc[connected_boards_df_one['symbol'] == symbol, 'connected_boards_numbers'] = \
            list(connected_boards_df_one['connected_boards_numbers'])[0]

    return stock_em_zt_pool_df_data


def sync_miss_zt_data(stock_em_zt_pool_df_data, str_day):
    now_date = datetime.now()
    now_day = now_date.strftime('%Y-%m-%d')
    if now_day == str_day:
        real_time_quotes_all_stocks_df = east_money_stock_api.get_real_time_quotes_all_stocks()
        real_time_quotes_all_stocks_df = real_time_quotes_all_stocks_df.loc[
            (real_time_quotes_all_stocks_df['wei_bi'] == 100) & (real_time_quotes_all_stocks_df['chg'] >= 9)]
        miss_zt_data_df = real_time_quotes_all_stocks_df.loc[~(
            real_time_quotes_all_stocks_df['symbol'].isin(stock_em_zt_pool_df_data['symbol']))]
        if data_frame_util.is_not_empty(miss_zt_data_df):
            miss_zt_data_df['buy_1_num'] = miss_zt_data_df['buy_1_num'].astype(float)
            miss_zt_data_df['now_price'] = miss_zt_data_df['now_price'].astype(float)
            miss_zt_data_df['closure_funds'] = round(miss_zt_data_df['buy_1_num'] * 100 * miss_zt_data_df['now_price'],
                                                     2)
            miss_zt_data_df = miss_zt_data_df[[
                'symbol',
                'name',
                'chg',
                'now_price',
                'amount',
                'flow_mv',
                'total_mv',
                'exchange',
                'industry',
                'closure_funds'

            ]]
            miss_zt_data_df['index'] = 10000
            miss_zt_data_df['first_closure_time'] = '150000'
            miss_zt_data_df['last_closure_time'] = '150000'
            miss_zt_data_df['statistics'] = '1/1'
            miss_zt_data_df['frying_plates_numbers'] = 0
            miss_zt_data_df['connected_boards_numbers'] = 0

            stock_em_zt_pool_df_data = pd.concat([miss_zt_data_df, stock_em_zt_pool_df_data])
        return stock_em_zt_pool_df_data


if __name__ == '__main__':
    save_zt_info('2024-10-30')
# from datetime import datetime
#
# if __name__ == '__main__':
#
#     sync_date = date_handle_util.add_date_day('20240110', 0)
#
#     now_date = datetime.now()
#
#     str_now_day = sync_date.strftime('%Y-%m-%d')
#
#     while now_date > sync_date:
#         try:
#             save_zt_info(str_now_day)
#             sync_date = date_handle_util.add_date_day(date_handle_util.no_slash_date(str_now_day), 1)
#             print(str_now_day)
#             str_now_day = sync_date.strftime('%Y-%m-%d')
#
#         except BaseException as e:
#             sync_date = date_handle_util.add_date_day(date_handle_util.no_slash_date(str_now_day), 1)
#             str_now_day = sync_date.strftime('%Y-%m-%d')



import sys
import os

file_path = os.path.abspath(__file__)
end = file_path.index('mns') + 16
project_path = file_path[0:end]
sys.path.append(project_path)
from datetime import datetime

import mns_common.constant.db_name_constant as db_name_constant
import mns_scheduler.finance.em_financial_profit_sync_service_api as em_financial_profit_sync_service_api
from mns_common.db.MongodbUtil import MongodbUtil
from loguru import logger
import \
    mns_scheduler.finance.em_financial_asset_liability_sync_service_api as em_financial_asset_liability_sync_service_api
import mns_scheduler.risk.financial_report_risk_check_api as financial_report_risk_check_api
import mns_common.utils.data_frame_util as data_frame_util
import mns_scheduler.finance.finance_common_api as finance_common_api
import mns_scheduler.risk.compliance.undisclosed_annual_report_api as undisclosed_annual_report_api

mongodb_util = MongodbUtil('27017')


# 上市公司年报披露时间:每年1月1日一- 4月30日。
# 2、上市公司中年报披露时间:每年7月1日--8月30日。
# 3、上市公司季报披露时间:
#    1季报:每年4月1日-- -4月30日。
#    2季报(中报) :每年7月1日--8月30日。
#    3季报:每年10月1日--10月31日4季报(年报) :每年1月1日--4月30日

def sync_financial_report():
    now_date = datetime.now()
    now_year = now_date.year
    now_month = now_date.month
    sync_time = now_date.strftime('%Y-%m-%d %H:%M:%S')
    # 年报
    if 1 <= now_month <= 5:
        period = 4
        period_time = str(now_year - 1) + "-12-31 00:00:00"
        sync_profit_report(period_time, sync_time, period)
        sync_asset_liability_report(period_time, sync_time, period)

    # 一季报
    elif now_month == 5:
        period = 1
        period_time = str(now_year) + "-03-31 00:00:00"
        sync_profit_report(period_time, sync_time, period)
        sync_asset_liability_report(period_time, sync_time, period)

    # 二季报
    elif 7 <= now_month <= 8:
        period = 2
        period_time = str(now_year) + "-06-30 00:00:00"
        sync_profit_report(period_time, sync_time, period)
        sync_asset_liability_report(period_time, sync_time, period)
    # 三季报
    elif now_month == 10:
        period = 3
        period_time = str(now_year) + "-09-30 00:00:00"
        sync_profit_report(period_time, sync_time, period)
        sync_asset_liability_report(period_time, sync_time, period)
    # 未出报告check
    undisclosed_annual_report_api.un_disclosed_report_check(sync_time, now_year, period, period_time)
    # 新股或者未出报告的
    sync_miss_report(sync_time)


# 同步资产表
def sync_asset_liability_report(period_time, sync_time, period):
    un_report_asset_df = finance_common_api.find_un_report_symbol(period_time,
                                                                  db_name_constant.EM_STOCK_ASSET_LIABILITY)
    for un_report_asset_one in un_report_asset_df.itertuples():
        try:
            symbol = un_report_asset_one.symbol
            new_asset_df = em_financial_asset_liability_sync_service_api.get_em_asset_liability_api(symbol)
            # 负债比
            new_asset_df['liability_ratio'] = round(
                new_asset_df['TOTAL_LIABILITIES'] * 100 / new_asset_df['TOTAL_ASSETS'],
                2)
            new_asset_df['sync_time'] = sync_time
            if data_frame_util.is_empty(new_asset_df):
                continue
            new_asset_df['symbol'] = symbol
            mongodb_util.insert_mongo(new_asset_df, db_name_constant.EM_STOCK_ASSET_LIABILITY)

            # 年报审核
            financial_report_risk_check_api.financial_report_check(new_asset_df, period_time, period,
                                                                   db_name_constant.EM_STOCK_ASSET_LIABILITY)

        except Exception as e:
            logger.error("同步资产表异常:{},{},{}", symbol, period_time, e)


# 同步利润表
def sync_profit_report(period_time, sync_time, period):
    un_report_profit_df = finance_common_api.find_un_report_symbol(period_time, db_name_constant.EM_STOCK_PROFIT)
    for un_report_profit_one in un_report_profit_df.itertuples():
        try:
            symbol = un_report_profit_one.symbol
            new_profit_df = em_financial_profit_sync_service_api.get_em_profit_api(symbol)
            if data_frame_util.is_empty(new_profit_df):
                continue
            new_profit_df['sync_time'] = sync_time

            new_profit_df['symbol'] = symbol
            mongodb_util.insert_mongo(new_profit_df, db_name_constant.EM_STOCK_PROFIT)

            # 年报审核
            financial_report_risk_check_api.financial_report_check(new_profit_df, period_time,
                                                                   period, db_name_constant.EM_STOCK_PROFIT)
        except Exception as e:
            logger.error("同步利润表异常:{},{},{}", symbol, period_time, e)


def sync_miss_report(sync_time):
    query = {"total_operate_income": 0}
    un_report_company_info = mongodb_util.find_query_data(db_name_constant.COMPANY_INFO, query)
    for un_report_one in un_report_company_info.itertuples():
        try:
            symbol = un_report_one.symbol
            new_profit_df = em_financial_profit_sync_service_api.get_em_profit_api(symbol)
            if data_frame_util.is_empty(new_profit_df):
                continue
            new_profit_df['sync_time'] = sync_time

            new_profit_df['symbol'] = symbol
            mongodb_util.insert_mongo(new_profit_df, db_name_constant.EM_STOCK_PROFIT)

            new_asset_df = em_financial_asset_liability_sync_service_api.get_em_asset_liability_api(symbol)
            # 负债比
            new_asset_df['liability_ratio'] = round(
                new_asset_df['TOTAL_LIABILITIES'] * 100 / new_asset_df['TOTAL_ASSETS'],
                2)
            new_asset_df['sync_time'] = sync_time
            if data_frame_util.is_empty(new_asset_df):
                continue
            new_asset_df['symbol'] = symbol
            mongodb_util.insert_mongo(new_asset_df, db_name_constant.EM_STOCK_ASSET_LIABILITY)

        except Exception as e:
            logger.error("同步财报补偿任务异常:{},{},{}", symbol, e)


if __name__ == '__main__':
    now_date = datetime.now()
    now_year = now_date.year
    now_month = now_date.month
    sync_time_test = now_date.strftime('%Y-%m-%d %H:%M:%S')
    sync_miss_report(sync_time_test)
